from typing import Optional, TypeVar, Iterable

import numpy as np
import pandas as pd


# TODO: This should be a generic over the type of the values, but there is not 
# good support currently for numpy and pandas type annotations 
# we should investigate numpy and pandas typing support and migrate
# https://github.com/numpy/numpy-stubs
# https://github.com/pandas-dev/pandas/blob/master/pandas/_typing.py 
ArrayLike = TypeVar("ArrayLike", list, np.ndarray, pd.Series, tuple)


class EcephysProjectApi:
    def get_sessions(
        self,
        session_ids: Optional[ArrayLike] = None,
        published_at: Optional[str] = None
    ):
        raise NotImplementedError()

    def get_session_data(self, session_id: int) -> Iterable:
        raise NotImplementedError()

    def get_isi_experiments(self, *args, **kwargs):
        raise NotImplementedError()

    def get_units(
        self, 
        unit_ids: Optional[ArrayLike] = None, 
        channel_ids: Optional[ArrayLike] = None, 
        probe_ids: Optional[ArrayLike] = None, 
        session_ids: Optional[ArrayLike] = None, 
        published_at: Optional[str] = None
    ):
        raise NotImplementedError()

    def get_channels(
        self, 
        channel_ids: Optional[ArrayLike] = None, 
        probe_ids: Optional[ArrayLike] = None, 
        session_ids: Optional[ArrayLike] = None, 
        published_at: Optional[str] = None
    ):
        raise NotImplementedError()

    def get_probes(
        self, 
        probe_ids: Optional[ArrayLike] = None, 
        session_ids: Optional[ArrayLike] = None, 
        published_at: Optional[str] = None
    ):
        raise NotImplementedError()

    def get_probe_lfp_data(self, probe_id: int) -> Iterable:
        raise NotImplementedError()

    def get_natural_movie_template(self, number) -> Iterable:
        raise NotImplementedError()

    def get_natural_scene_template(self, number) -> Iterable:
        raise NotImplementedError()

    def get_unit_analysis_metrics(
        self, 
        unit_ids: Optional[ArrayLike] = None, 
        ecephys_session_ids: Optional[ArrayLike] = None, 
        session_types: Optional[ArrayLike] = None
    ) -> pd.DataFrame:
        raise NotImplementedError()