import pandas as pd

from allensdk.brain_observatory.argschema_utilities import \
    ArgSchemaParserPlus, \
    write_or_print_outputs
from allensdk.brain_observatory.ecephys.file_io.ecephys_sync_dataset import (
    EcephysSyncDataset,
)
from ._schemas import InputParameters, OutputParameters


def build_opto_table(args):
    opto_file = pd.read_pickle(args['opto_pickle_path'])
    sync_file = EcephysSyncDataset.factory(args['sync_h5_path'])

    start_times = sync_file.extract_led_times()
    conditions = [str(item) for item in opto_file['opto_conditions']]
    levels = opto_file['opto_levels']

    assert len(conditions) == len(levels)
    if len(start_times) > len(conditions):
        raise ValueError(
            f"there are {len(start_times) - len(conditions)} extra "
            f"optotagging sync times!")

    optotagging_table = pd.DataFrame({
        'start_time': start_times,
        'condition': conditions,
        'level': levels
    })
    optotagging_table = optotagging_table.sort_values(by='start_time', axis=0)

    stop_times = []
    names = []
    conditions = []
    for ii, row in optotagging_table.iterrows():
        condition = args["conditions"][row["condition"]]
        stop_times.append(row["start_time"] + condition["duration"])
        names.append(condition["name"])
        conditions.append(condition["condition"])

    optotagging_table["stop_time"] = stop_times
    optotagging_table["stimulus_name"] = names
    optotagging_table["condition"] = conditions
    optotagging_table["duration"] = \
        optotagging_table["stop_time"] - optotagging_table["start_time"]

    optotagging_table.to_csv(args['output_opto_table_path'], index=False)
    return {'output_opto_table_path': args['output_opto_table_path']}


def main():
    mod = ArgSchemaParserPlus(schema_type=InputParameters,
                              output_schema_type=OutputParameters)
    output = build_opto_table(mod.args)

    write_or_print_outputs(data=output, parser=mod)


if __name__ == "__main__":
    main()
